local AceGUI = LibStub("AceGUI-3.0")

--[[
This is a from-scratch AceGUI-style implementation of the combination drop-
down editboxes seen in DoTimer's option menus.  Typing in the editbox adds
entries to a list; clicking the scrolldown button displays the list in a
dropdown; clicking on an entry in the dropdown removes it from the list.

(This type of widget may actually have a more formal name already.  I first
encountered it in DoTimer, so that's what I named it after.)

The implementation does not borrow any of DoTimer's actual code.  I've tried
writing this to imitate the look-and-feel of an interface that many players
will find familiar and easy to use.


EditBoxDropDown API

:SetLabel(txt)
   forwards to the editbox's SetLabel

:SetText(txt)
   forwards to the editbox's SetText

:SetEditBoxTooltip(txt)
   sets text for the tooltip shown when hovering over the editbox
   no default

:SetButtonTooltip(txt)
   sets text for the tooltip shown when hovering over the dropdown button
   default "Click on entries to remove them."

:SetList(t)
   T is a table to be shown in the dropdown list; the values will be displayed
   in the dropdown
   When entries are clicked, they will be removed from T.  T is not copied,
   the table is edited "live" before firing the callback below.


EditBoxDropDown Callbacks

OnTextEnterPressed
   same as the editbox's OnEnterPressed

OnListItemClicked
   similar to a Dropdown widget's OnValueChanged, the key and value from the
   table given to :SetList are passed


farmbuyer
]]
do
	local Type = "EditBoxDropDown"
	local Version = 1

	local button_hover_text_default = "Click on entries to remove them."
	local maps

	local function OnAcquire(self)
		self:SetHeight(20)
		self:SetWidth(126)
		self.button.tooltip_text = button_hover_text_default
		self:SetList(nil)

		maps = maps or {}
		maps[self.editbox.editbox] = self
	end

	local function OnRelease(self)
		self.frame:ClearAllPoints()
		self.frame:Hide()
		self.editbox.tooltip_text = nil
		self.button.tooltip_text = nil
		self:SetList(nil)
		--AceGUI:Release(self.editbox)
		maps[self.editbox.editbox] = nil
	end

	local function SetParent (self, parent)
		self.frame:SetParent(nil)
		self.frame:SetParent(parent.content)
		self.parent = parent
		self.editbox:SetParent(parent)
	end

	local function Menu_OnClick (button, userlist_key, widget)
		local v = widget.list[userlist_key]
		widget.list[userlist_key] = nil
		widget:Fire("OnListItemClicked", userlist_key, v)
	end

	local function BuildList (self)
		local ret = {}
		if self.list then for k,v in pairs(self.list) do
			table.insert (ret, {
				text = tostring(v) or tostring(k),
				func = Menu_OnClick,
				arg1 = k,
				arg2 = self,
				notCheckable = true,
			})
		end end
		return ret
	end

	local function SetText (self, text)
		self.editbox_basetext = text
		return self.editbox:SetText(text)
	end

	local function SetLabel (self, text)
		return self.editbox:SetLabel(text)
	end

	local function SetList (self, list)
		self.list = list
	end

	local function SetEditBoxTooltip (self, text)
		self.editbox.tooltip_text = text
	end
	local function SetButtonTooltip (self, text)
		self.button.tooltip_text = text
	end

	local function ddEditBox_OnMouseEnter(self)
		if self.tooltip_text then
			GameTooltip:SetOwner(self.frame, "ANCHOR_RIGHT")
			GameTooltip:SetText(self.tooltip_text, nil, nil, nil, nil, 1)
		end
	end

	local function ddEditBox_OnMouseLeave(self)
		GameTooltip:Hide()
	end

	local function ddEditBox_Clear (self)
		self:SetText("")
	end

	local function ddEditBox_Reset (self)  -- :ClearFocus triggers this
		self:SetText(maps[self].editbox_basetext)
	end

	local function ddEditBox_OnEnterPressed (self, _, text)
		self.obj:Fire("OnTextEnterPressed", text)
		ddEditBox_Reset(self.editbox)
		self.editbox:ClearFocus()  -- cursor still blinking in there, one more time
	end

	local function Button_OnClick(self)
		local t = BuildList(self.obj)
		EasyMenu (t, self.obj.dropdown, self.obj.frame, 0, 0, "MENU")
		PlaySound("igMainMenuOptionCheckBoxOn")
	end

	local function Button_OnEnter(self)
		if self.tooltip_text then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetText(self.tooltip_text, nil, nil, nil, nil, 1)
		end
	end


	local function Constructor()
		local num = AceGUI:GetNextWidgetNum(Type)
		local frame = CreateFrame("Frame","AceGUI-3.0EditBoxDropDown"..num,UIParent)
		local self = {}
		self.type = Type

		self.OnAcquire = OnAcquire
		self.OnRelease = OnRelease
		self.SetParent = SetParent
		self.SetText = SetText
		self.SetLabel = SetLabel
		self.SetList = SetList
		self.SetEditBoxTooltip = SetEditBoxTooltip

		self.frame = frame
		frame.obj = self

		frame:SetHeight(20)
		frame:SetWidth(126)

		local editbox = AceGUI:Create("EditBox")
		editbox:SetParent(frame)   -- also sets .parent
		editbox:SetHeight(20)
		editbox:SetWidth(100)
		editbox:SetPoint("TOPLEFT", frame, "TOPLEFT")
		editbox:SetCallback("OnEnter", ddEditBox_OnMouseEnter)
		editbox:SetCallback("OnLeave", ddEditBox_OnMouseLeave)
		editbox:SetCallback("OnEnterPressed", ddEditBox_OnEnterPressed)
		editbox.editbox:SetScript("OnEditFocusGained", ddEditBox_Clear)
		editbox.editbox:SetScript("OnEditFocusLost", ddEditBox_Reset)
		--editbox.editbox:SetScript("OnEscapePressed", ddEditBox_Reset)
		editbox.showbutton = false
		self.editbox = editbox
		editbox.obj = self

		local button = CreateFrame("Button", nil, frame)
		button:SetHeight(20)
		button:SetWidth(24)
		button:SetPoint("LEFT", editbox.frame, "RIGHT", 0, 0)
		button:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
		button:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
		button:SetDisabledTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Disabled")
		button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD")
		button:SetScript("OnClick", Button_OnClick)
		button:SetScript("OnEnter", Button_OnEnter)
		button:SetScript("OnLeave", ddEditBox_OnMouseLeave)
		self.button = button
		button.parent = frame
		button.obj = self

		local dropdown = CreateFrame("Frame", "AceGUI-3.0EditBoxDropDownMenu"..num, nil, "UIDropDownMenuTemplate")
		self.dropdown = dropdown
		dropdown.obj = self

		AceGUI:RegisterAsWidget(self)
		return self
	end

	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end

